/*
  use     : emu9520.exe [baudrate]
	   (1) must run on 9188
	   (2) default baudrate is 9600 (for COM2)
  function description:
	   (1) Transfer any data coming from 9188's COM2(RS485) to COM4.
	   (2) data coming from COM4(end with CR(0x0d)) will be send to
	       COM2(RS485)(data in will be buffered,until receive 0x0d,
	       9188 will send out the buffered data.(if checksum enable,
	       will append checksum(2 bytes) after the data,before 0x0d.)
	   (3) press 'q'(not 'Q') to quit.
	   (4) press 's'(not 'S') to toggle checksum mode.
	       so 'q' and 's' will not be send to RS485.
	   (5) default mode is checksum disable.
	   (6) The protocol is 1 start bit,8 data bit,none parity,1 stop bit.

   [PC]<---------->[9188]<------------------------>[9000's]
	RS232  (COM4)    (COM2)  RS485

   NOTE: This program can do something 9520 cannot do,that is the baudrate
	 or protocol on RS232 and RS-485 can be asynchronous.
	 9520's RS232 and RS-485 must have the same baudrate and protocol,
	 and they can be changed anytime.
	 If users want to use PC to control 9000's with 9188, this program can
	 be used to emulate the use of 9520.
*/
#include<stdlib.h>
#include<stdio.h>
#include<string.h>
#include"..\lib\9188d.h"

unsigned no;
unsigned char buf[80];
unsigned char OutBuf[80];
int idx=0;
int outidx=0;
char h_to_a[16] ={
 '0','1','2','3','4','5','6','7',
 '8','9','A','B','C','D','E','F'
};
/*
void ToCOM4Str(char *s){
 int length,i;
	length=strlen(s);
	for(i=0;i<length;i++)
	  ToCom4(s[i]);
}
  */
void main(int argc,char *argv[])
{
  unsigned long time,sec;
  long baud=9600;
  int quit=0;
  int data;
  int i;
  int checksum=0;

  if(argc>1){
     baud=atol(argv[1]);
     if(baud<=0) baud=115200;
  }

  printf("\nUse COM2(RS485) connect to 9000's");
  printf("\nEMU9520 [COM2baudrate] -->  default:9600");
  printf("\nPress 's' to toggle checksum ON(1)/OFF(0)");
  printf("\nPress 'q' to quit");
  printf("\nCurrent mode:baudrate=%ld,checksum=%d\n",baud,checksum);
  InstallCom(2,baud,8,0,1);
  InstallCom(4,9600L,8,0,1); /* COM4 use fixed baudrate 9600 */
  Init5DigitLed();
  //Show "9520" on the 5-digit LED
  Show5DigitLed(5,16);
  Show5DigitLed(4,0);
  Show5DigitLed(3,2);
  Show5DigitLed(2,5);
  Show5DigitLed(1,9);

  while(!quit){
     if(IsCom(4)){
	data=ReadCom(4);
	if(data=='q') quit=1;
	else if(data=='s'){
	   checksum=!checksum;
	   ToCom4Str("CheckSum=");
	   ToCom4('0'+checksum);
	   ToCom4Str("\r\n");
	}
	else if(data=='\b'){
	      if(outidx){
		 outidx--;
		 ToCom(4,data);
	      }
	}
	else {
	   OutBuf[outidx++]=data;
	   /*ToCom(4,data);*/
	   if(data=='\r'){
	      unsigned char sum=0;
	      /*ToCom(4,'\n');*/
	      outidx--;
	      Set485DirToTransmit(2);
	      for(i=0;i<outidx;i++){
		ToCom(2,OutBuf[i]);
		if(checksum) sum+=OutBuf[i];
	      }
	      if(checksum){
		ToCom(2,h_to_a[sum>>4]);
		ToCom(2,h_to_a[sum&0xf]);

	      }
	      ToCom(2,'\r');
	      outidx=0;
	      WaitTransmitOver(2);
	      Set485DirToReceive(2);
	   }
	}
     }
     if(IsCom(2)){
	data=ReadCom(2);
	ToCom(4,data);
	buf[idx++]=data;
	if(data=='\r'){
	   idx=0;
	}
	else {
	   if(idx>=80) idx=79;
	}
     }
  }
  RestoreCom(2);
  RestoreCom(4);
}
